@@ -1,11 +1,12 @@
 from .checks import geo_query
+from .checks.base import AllEqualCheck
 from .checks.base import BaseCheck
 from .checks.base import CheckLevel
-from .checks.base import FileExistsCheck
 from .checks.base import ForeignKeyCheck
 from .checks.base import NotNullCheck
 from .checks.base import QueryCheck
 from .checks.base import RangeCheck
+from .checks.base import UniqueCheck
 from .checks.cross_section_definitions import CrossSectionEqualElementsCheck
 from .checks.cross_section_definitions import (
     CrossSectionFirstElementNonZeroCheck,
@@ -29,7 +30,16 @@ from .checks.other import ConnectionNodesLength
 from .checks.other import CrossSectionLocationCheck
 from .checks.other import LinestringLocationCheck
 from .checks.other import OpenChannelsWithNestedNewton
+from .checks.other import SpatialIndexCheck
 from .checks.other import Use0DFlowCheck
+from .checks.raster import GDALAvailableCheck
+from .checks.raster import RasterExistsCheck
+from .checks.raster import RasterHasOneBandCheck
+from .checks.raster import RasterHasProjectionCheck
+from .checks.raster import RasterIsProjectedCheck
+from .checks.raster import RasterIsValidCheck
+from .checks.raster import RasterRangeCheck
+from .checks.raster import RasterSquareCellsCheck
 from .checks.timeseries import TimeseriesIncreasingCheck
 from .checks.timeseries import TimeseriesRowCheck
 from .checks.timeseries import TimeseriesTimestepCheck
@@ -45,25 +55,61 @@ def is_none_or_empty(col):
     return (col == None) | (col == "")
 
 
+def scalar_subquery(query):
+    # compatibility between sqlalchemy 1.3 and 1.4
+    try:
+        return query.scalar_subquery()
+    except AttributeError:
+        return query.as_scalar()
+
+
+# Use these to make checks only work on the first global settings entry:
+first_setting = scalar_subquery(
+    Query(models.GlobalSetting.id).order_by(models.GlobalSetting.id).limit(1)
+)
+first_setting_filter = models.GlobalSetting.id == first_setting
+interflow_settings_id = scalar_subquery(
+    Query(models.GlobalSetting.interflow_settings_id).filter(first_setting_filter)
+)
+interflow_filter = models.Interflow.id == interflow_settings_id
+infiltration_settings_id = scalar_subquery(
+    Query(models.GlobalSetting.simple_infiltration_settings_id).filter(
+        first_setting_filter
+    )
+)
+infiltration_filter = models.SimpleInfiltration.id == infiltration_settings_id
+groundwater_settings_id = scalar_subquery(
+    Query(models.GlobalSetting.groundwater_settings_id).filter(first_setting_filter)
+)
+groundwater_filter = models.GroundWater.id == groundwater_settings_id
+
 CONDITIONS = {
     "has_dem": Query(models.GlobalSetting).filter(
-        ~is_none_or_empty(models.GlobalSetting.dem_file)
+        first_setting_filter, ~is_none_or_empty(models.GlobalSetting.dem_file)
     ),
     "has_no_dem": Query(models.GlobalSetting).filter(
-        is_none_or_empty(models.GlobalSetting.dem_file)
+        first_setting_filter, is_none_or_empty(models.GlobalSetting.dem_file)
     ),
     "0d_surf": Query(models.GlobalSetting).filter(
-        models.GlobalSetting.use_0d_inflow == constants.InflowType.SURFACE
+        first_setting_filter,
+        models.GlobalSetting.use_0d_inflow == constants.InflowType.SURFACE,
     ),
     "0d_imp": Query(models.GlobalSetting).filter(
-        models.GlobalSetting.use_0d_inflow == constants.InflowType.IMPERVIOUS_SURFACE
+        first_setting_filter,
+        models.GlobalSetting.use_0d_inflow == constants.InflowType.IMPERVIOUS_SURFACE,
+    ),
+    "manning": Query(models.GlobalSetting).filter(
+        first_setting_filter,
+        models.GlobalSetting.frict_type == constants.FrictionType.MANNING,
+    ),
+    "chezy": Query(models.GlobalSetting).filter(
+        first_setting_filter,
+        models.GlobalSetting.frict_type == constants.FrictionType.CHEZY,
     ),
 }
 
-try:
-    kmax = Query(models.GlobalSetting.kmax).limit(1).scalar_subquery()
-except AttributeError:
-    kmax = Query(models.GlobalSetting.kmax).limit(1).as_scalar()
+kmax = scalar_subquery(Query(models.GlobalSetting.kmax).filter(first_setting_filter))
+
 
 CHECKS: List[BaseCheck] = []
 
@@ -535,6 +581,11 @@ CHECKS += [
         message="The 'the_geom_linestring' column of v2_connection_nodes must be NULL",
     )
 ]
+CHECKS += [
+    SpatialIndexCheck(
+        error_code=207, column=models.ConnectionNode.the_geom, level=CheckLevel.WARNING
+    )
+]
 
 
 ## 025x: Connectivity
@@ -612,6 +663,33 @@ CHECKS += [
         models.Weir,
     )
 ]
+CHECKS += [
+    QueryCheck(
+        error_code=254,
+        level=CheckLevel.ERROR,
+        column=models.ConnectionNode.id,
+        invalid=Query(models.ConnectionNode)
+        .join(models.Manhole, isouter=True)
+        .filter(
+            models.Manhole.bottom_level == None,
+            models.ConnectionNode.id.notin_(
+                Query(models.Pipe.connection_node_start_id).union_all(
+                    Query(models.Pipe.connection_node_end_id),
+                    Query(models.Channel.connection_node_start_id),
+                    Query(models.Channel.connection_node_end_id),
+                    Query(models.Culvert.connection_node_start_id),
+                    Query(models.Culvert.connection_node_end_id),
+                    Query(models.Weir.connection_node_start_id),
+                    Query(models.Weir.connection_node_end_id),
+                    Query(models.Orifice.connection_node_start_id),
+                    Query(models.Orifice.connection_node_end_id),
+                )
+            ),
+        ),
+        message="A connection node that is not connected to a pipe, "
+        "channel, culvert, weir, or orifice must have a manhole with a bottom_level.",
+    ),
+]
 
 
 ## 030x: SETTINGS
@@ -621,6 +699,7 @@ CHECKS += [
         error_code=302,
         column=models.GlobalSetting.dem_obstacle_detection,
         invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
             models.GlobalSetting.dem_obstacle_detection == True,
         ),
         message="v2_global_settings.dem_obstacle_detection is True, while this feature is not supported",
@@ -630,6 +709,7 @@ CHECKS += [
         level=CheckLevel.WARNING,
         column=models.GlobalSetting.use_1d_flow,
         invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
             models.GlobalSetting.use_1d_flow == False,
             Query(func.count(models.ConnectionNode.id) > 0).label("1d_count"),
         ),
@@ -640,6 +720,7 @@ CHECKS += [
         error_code=304,
         column=models.GlobalSetting.groundwater_settings_id,
         invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
             models.GlobalSetting.groundwater_settings_id != None,
             models.GlobalSetting.simple_infiltration_settings != None,
         ),
@@ -648,6 +729,7 @@ CHECKS += [
     RangeCheck(
         error_code=305,
         column=models.GlobalSetting.kmax,
+        filters=first_setting_filter,
         min_value=0,
         left_inclusive=False,  # 0 is not allowed
     ),
@@ -655,6 +737,7 @@ CHECKS += [
         error_code=306,
         level=CheckLevel.WARNING,
         column=models.GlobalSetting.dist_calc_points,
+        filters=first_setting_filter,
         min_value=0,
         left_inclusive=False,  # 0 itself is not allowed
         message="v2_global_settings.dist_calc_points is not greater than 0, in the future this will lead to an error",
@@ -662,53 +745,60 @@ CHECKS += [
     RangeCheck(
         error_code=307,
         column=models.GlobalSetting.grid_space,
+        filters=first_setting_filter,
         min_value=0,
         left_inclusive=False,  # 0 itself is not allowed
     ),
     RangeCheck(
         error_code=308,
         column=models.GlobalSetting.embedded_cutoff_threshold,
+        filters=first_setting_filter,
         min_value=0,
     ),
     RangeCheck(
         error_code=309,
         column=models.GlobalSetting.max_angle_1d_advection,
+        filters=first_setting_filter,
         min_value=0,
         max_value=0.5 * 3.14159,
     ),
     RangeCheck(
         error_code=310,
         column=models.GlobalSetting.table_step_size,
+        filters=first_setting_filter,
         min_value=0,
         left_inclusive=False,
     ),
     RangeCheck(
         error_code=311,
         column=models.GlobalSetting.table_step_size_1d,
+        filters=first_setting_filter,
         min_value=0,
         left_inclusive=False,
     ),
     RangeCheck(
         error_code=313,
         column=models.GlobalSetting.frict_coef,
-        filters=models.GlobalSetting.frict_type == constants.FrictionType.MANNING,
+        filters=CONDITIONS["manning"].exists(),
         min_value=0,
         max_value=1,
     ),
     RangeCheck(
         error_code=314,
         column=models.GlobalSetting.frict_coef,
-        filters=models.GlobalSetting.frict_type == constants.FrictionType.CHEZY,
+        filters=CONDITIONS["chezy"].exists(),
         min_value=0,
     ),
     RangeCheck(
         error_code=315,
         column=models.GlobalSetting.interception_global,
+        filters=first_setting_filter,
         min_value=0,
     ),
     RangeCheck(
         error_code=316,
         column=models.GlobalSetting.manhole_storage_area,
+        filters=first_setting_filter,
         min_value=0,
     ),
     QueryCheck(
@@ -736,6 +826,7 @@ CHECKS += [
         error_code=320,
         column=models.GlobalSetting.use_2d_flow,
         invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
             models.GlobalSetting.use_1d_flow == False,
             models.GlobalSetting.use_2d_flow == False,
         ),
@@ -746,6 +837,7 @@ CHECKS += [
         error_code=321,
         column=models.GlobalSetting.manhole_storage_area,
         invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
             models.GlobalSetting.manhole_storage_area > 0,
             (
                 (models.GlobalSetting.use_2d_flow == True)
@@ -758,6 +850,7 @@ CHECKS += [
         error_code=322,
         column=models.GlobalSetting.water_level_ini_type,
         invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
             ~is_none_or_empty(models.GlobalSetting.initial_waterlevel_file),
             models.GlobalSetting.water_level_ini_type == None,
         ),
@@ -767,11 +860,61 @@ CHECKS += [
         error_code=323,
         column=models.GlobalSetting.maximum_table_step_size,
         invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
             models.GlobalSetting.maximum_table_step_size
             < models.GlobalSetting.table_step_size,
         ),
         message="v2_global_settings.maximum_table_step_size should be greater than v2_global_settings.table_step_size.",
     ),
+    UniqueCheck(
+        error_code=324,
+        level=CheckLevel.WARNING,
+        columns=(
+            models.AggregationSettings.flow_variable,
+            models.AggregationSettings.aggregation_method,
+        ),
+    ),
+    QueryCheck(
+        error_code=325,
+        level=CheckLevel.WARNING,
+        column=models.GlobalSetting.interception_global,
+        invalid=Query(models.GlobalSetting).filter(
+            first_setting_filter,
+            ~is_none_or_empty(models.GlobalSetting.interception_file),
+        ),
+        message="v2_global_settings.interception_global is recommended as fallback value when using an interception_file.",
+    ),
+]
+
+CHECKS += [
+    AllEqualCheck(error_code=330 + i, column=column, level=CheckLevel.WARNING)
+    for i, column in enumerate(
+        [
+            models.GlobalSetting.use_2d_flow,
+            models.GlobalSetting.use_1d_flow,
+            models.GlobalSetting.grid_space,
+            models.GlobalSetting.dist_calc_points,
+            models.GlobalSetting.kmax,
+            models.GlobalSetting.dem_file,
+            models.GlobalSetting.embedded_cutoff_threshold,
+            models.GlobalSetting.epsg_code,
+            models.GlobalSetting.max_angle_1d_advection,
+            models.GlobalSetting.frict_avg,
+            models.GlobalSetting.use_0d_inflow,
+            models.GlobalSetting.manhole_storage_area,
+            models.GlobalSetting.table_step_size,
+            models.GlobalSetting.frict_type,
+            models.GlobalSetting.frict_coef,
+            models.GlobalSetting.frict_coef_file,
+            models.GlobalSetting.interception_global,
+            models.GlobalSetting.interception_file,
+            models.GlobalSetting.table_step_size_1d,
+            models.GlobalSetting.maximum_table_step_size,
+            models.GlobalSetting.interflow_settings_id,
+            models.GlobalSetting.simple_infiltration_settings_id,
+            models.GlobalSetting.groundwater_settings_id,
+        ]
+    )
 ]
 
 ## 04xx: Groundwater, Interflow & Infiltration
@@ -779,71 +922,100 @@ CHECKS += [
     RangeCheck(
         error_code=401,
         column=models.Interflow.porosity,
-        filters=models.Interflow.global_settings != None,
+        filters=interflow_filter,
         min_value=0,
         max_value=1,
     ),
     RangeCheck(
         error_code=402,
         column=models.Interflow.impervious_layer_elevation,
-        filters=models.Interflow.global_settings != None,
+        filters=interflow_filter,
         min_value=0,
     ),
     RangeCheck(
         error_code=403,
         column=models.SimpleInfiltration.infiltration_rate,
-        filters=models.SimpleInfiltration.global_settings != None,
+        filters=infiltration_filter,
         min_value=0,
     ),
     QueryCheck(
         error_code=404,
         column=models.SimpleInfiltration.infiltration_rate,
         invalid=Query(models.SimpleInfiltration).filter(
-            models.SimpleInfiltration.global_settings != None,
+            infiltration_filter,
             models.SimpleInfiltration.infiltration_rate == None,
             is_none_or_empty(models.SimpleInfiltration.infiltration_rate_file),
         ),
-        message="a global simple infiltration rate (v2_simple_infiltration.infiltration_rate) should be defined when not using a simple infiltration rate file.",
+        message="v2_simple_infiltration.infiltration_rate must be defined.",
+    ),
+    QueryCheck(
+        error_code=404,
+        level=CheckLevel.WARNING,
+        column=models.SimpleInfiltration.infiltration_rate,
+        invalid=Query(models.SimpleInfiltration).filter(
+            infiltration_filter,
+            models.SimpleInfiltration.infiltration_rate == None,
+            ~is_none_or_empty(models.SimpleInfiltration.infiltration_rate_file),
+        ),
+        message="v2_simple_infiltration.infiltration_rate is recommended as fallback value when using an infiltration_rate_file.",
     ),
     QueryCheck(
         error_code=405,
         column=models.GroundWater.equilibrium_infiltration_rate,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
-            (models.GroundWater.equilibrium_infiltration_rate == None)
-            | (models.GroundWater.equilibrium_infiltration_rate < 0),
-            is_none_or_empty(
-                models.GroundWater.equilibrium_infiltration_rate_file,
-            ),
+            groundwater_filter,
+            models.GroundWater.equilibrium_infiltration_rate == None,
+            is_none_or_empty(models.GroundWater.equilibrium_infiltration_rate_file),
         ),
-        message="a global equilibrium infiltration rate (v2_groundwater.equilibrium_infiltration_rate) should be defined and >=0 when not using an equilibrium infiltration rate file.",
+        message="v2_groundwater.equilibrium_infiltration_rate must be defined when not using an equilibrium_infiltration_rate_file.",
+    ),
+    QueryCheck(
+        error_code=405,
+        level=CheckLevel.WARNING,
+        column=models.GroundWater.equilibrium_infiltration_rate,
+        invalid=Query(models.GroundWater).filter(
+            groundwater_filter,
+            models.GroundWater.equilibrium_infiltration_rate == None,
+            ~is_none_or_empty(models.GroundWater.equilibrium_infiltration_rate_file),
+        ),
+        message="v2_groundwater.equilibrium_infiltration_rate is recommended as fallback value when using an equilibrium_infiltration_rate_file.",
     ),
     QueryCheck(
         error_code=406,
         column=models.GroundWater.equilibrium_infiltration_rate_type,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.equilibrium_infiltration_rate_type == None,
             ~is_none_or_empty(models.GroundWater.equilibrium_infiltration_rate_file),
         ),
-        message="an equilibrium infiltration rate type (v2_groundwater.equilibrium_infiltration_rate_type) should be defined when using an equilibrium infiltration rate file.",
+        message="v2_groundwater.equilibrium_infiltration_rate_type should be defined when using an equilibrium_infiltration_rate_file.",
     ),
     QueryCheck(
         error_code=407,
         column=models.GroundWater.infiltration_decay_period,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
-            (models.GroundWater.infiltration_decay_period == None)
-            | (models.GroundWater.infiltration_decay_period <= 0),
+            groundwater_filter,
+            models.GroundWater.infiltration_decay_period == None,
             is_none_or_empty(models.GroundWater.infiltration_decay_period_file),
         ),
-        message="a global infiltration decay period (v2_groundwater.infiltration_decay_period) should be defined and >0 when not using an infiltration decay period file.",
+        message="v2_groundwater.infiltration_decay_period must be defined when not using an infiltration_decay_period_file.",
+    ),
+    QueryCheck(
+        error_code=407,
+        level=CheckLevel.WARNING,
+        column=models.GroundWater.infiltration_decay_period,
+        invalid=Query(models.GroundWater).filter(
+            groundwater_filter,
+            models.GroundWater.infiltration_decay_period == None,
+            ~is_none_or_empty(models.GroundWater.infiltration_decay_period_file),
+        ),
+        message="v2_groundwater.infiltration_decay_period is recommended as fallback value when using an infiltration_decay_period_file.",
     ),
     QueryCheck(
         error_code=408,
         column=models.GroundWater.infiltration_decay_period_type,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.infiltration_decay_period_type == None,
             ~is_none_or_empty(models.GroundWater.infiltration_decay_period_file),
         ),
@@ -853,72 +1025,106 @@ CHECKS += [
         error_code=409,
         column=models.GroundWater.groundwater_hydro_connectivity_type,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.groundwater_hydro_connectivity_type == None,
             ~is_none_or_empty(models.GroundWater.groundwater_hydro_connectivity_file),
         ),
-        message="a groundwater hydro connectivity type (v2_groundwater.groundwater_hydro_connectivity_type) should be defined when using a groundwater hydro connectivity file.",
+        message="v2_groundwater.groundwater_hydro_connectivity_type should be defined when using a groundwater_hydro_connectivity_file.",
     ),
     QueryCheck(
         error_code=410,
         column=models.GroundWater.groundwater_impervious_layer_level,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.groundwater_impervious_layer_level == None,
             is_none_or_empty(
                 models.GroundWater.groundwater_impervious_layer_level_file
             ),
         ),
-        message="a global impervious layer level (v2_groundwater.groundwater_impervious_layer_level) should be defined when not using an impervious layer level file",
+        message="v2_groundwater.groundwater_impervious_layer_level must be defined when not using an groundwater_impervious_layer_level_file",
+    ),
+    QueryCheck(
+        error_code=410,
+        level=CheckLevel.WARNING,
+        column=models.GroundWater.groundwater_impervious_layer_level,
+        invalid=Query(models.GroundWater).filter(
+            groundwater_filter,
+            models.GroundWater.groundwater_impervious_layer_level == None,
+            ~is_none_or_empty(
+                models.GroundWater.groundwater_impervious_layer_level_file
+            ),
+        ),
+        message="v2_groundwater.groundwater_impervious_layer_level is recommended as fallback value when using a groundwater_impervious_layer_level_file.",
     ),
     QueryCheck(
         error_code=411,
         column=models.GroundWater.groundwater_impervious_layer_level_type,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.groundwater_impervious_layer_level_type == None,
             ~is_none_or_empty(
                 models.GroundWater.groundwater_impervious_layer_level_file
             ),
         ),
-        message="a impervious layer level type (v2_groundwater.groundwater_impervious_layer_level_type) should be defined when using an impervious layer level file",
+        message="v2_groundwater.groundwater_impervious_layer_level_type should be defined when using a groundwater_impervious_layer_level_file",
     ),
     QueryCheck(
         error_code=412,
         column=models.GroundWater.initial_infiltration_rate,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
-            (models.GroundWater.initial_infiltration_rate == None)
-            | (models.GroundWater.initial_infiltration_rate < 0),
+            groundwater_filter,
+            models.GroundWater.initial_infiltration_rate == None,
             is_none_or_empty(models.GroundWater.initial_infiltration_rate_file),
         ),
-        message="a global initial infiltration rate (v2_groundwater.initial_infiltration_rate) should be defined and >=0 when not using an initial infiltration rate file.",
+        message="v2_groundwater.initial_infiltration_rate must be defined when not using a initial_infiltration_rate_file.",
+    ),
+    QueryCheck(
+        error_code=412,
+        level=CheckLevel.WARNING,
+        column=models.GroundWater.initial_infiltration_rate,
+        invalid=Query(models.GroundWater).filter(
+            groundwater_filter,
+            models.GroundWater.initial_infiltration_rate == None,
+            ~is_none_or_empty(models.GroundWater.initial_infiltration_rate_file),
+        ),
+        message="v2_groundwater.initial_infiltration_rate is recommended as fallback value when using a initial_infiltration_rate_file.",
     ),
     QueryCheck(
         error_code=413,
         column=models.GroundWater.initial_infiltration_rate_type,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.initial_infiltration_rate_type == None,
             ~is_none_or_empty(models.GroundWater.initial_infiltration_rate_file),
         ),
-        message="a initial infiltration rate type (v2_groundwater.initial_infiltration_rate_type) should be defined when using an initial infiltration rate file.",
+        message="v2_groundwater.initial_infiltration_rate_type should be defined when using an initial infiltration rate file.",
     ),
     QueryCheck(
         error_code=414,
         column=models.GroundWater.phreatic_storage_capacity,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.phreatic_storage_capacity == None,
             is_none_or_empty(models.GroundWater.phreatic_storage_capacity_file),
         ),
-        message="a global phreatic storage capacity (v2_groundwater.phreatic_storage_capacity) should be defined when using a phreatic storage capacity file.",
+        message="v2_groundwater.phreatic_storage_capacity must be defined when not using a phreatic_storage_capacity_file.",
+    ),
+    QueryCheck(
+        error_code=414,
+        level=CheckLevel.WARNING,
+        column=models.GroundWater.phreatic_storage_capacity,
+        invalid=Query(models.GroundWater).filter(
+            groundwater_filter,
+            models.GroundWater.phreatic_storage_capacity == None,
+            ~is_none_or_empty(models.GroundWater.phreatic_storage_capacity_file),
+        ),
+        message="v2_groundwater.phreatic_storage_capacity is recommended as fallback value when using a phreatic_storage_capacity_file.",
     ),
     QueryCheck(
         error_code=415,
         column=models.GroundWater.phreatic_storage_capacity_type,
         invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
+            groundwater_filter,
             models.GroundWater.phreatic_storage_capacity_type == None,
             ~is_none_or_empty(models.GroundWater.phreatic_storage_capacity_file),
         ),
@@ -928,18 +1134,30 @@ CHECKS += [
         error_code=416,
         column=models.Interflow.porosity,
         invalid=Query(models.Interflow).filter(
-            models.Interflow.global_settings != None,
+            interflow_filter,
             models.Interflow.porosity == None,
             is_none_or_empty(models.Interflow.porosity_file),
             models.Interflow.interflow_type != constants.InterflowType.NO_INTERLFOW,
         ),
-        message="a global porosity (v2_interflow.porosity) should be defined when not using a porosity file.",
+        message="v2_interflow.porosity must be defined when not using a porosity_file.",
+    ),
+    QueryCheck(
+        error_code=416,
+        level=CheckLevel.WARNING,
+        column=models.Interflow.porosity,
+        invalid=Query(models.Interflow).filter(
+            interflow_filter,
+            models.Interflow.porosity == None,
+            ~is_none_or_empty(models.Interflow.porosity_file),
+            models.Interflow.interflow_type != constants.InterflowType.NO_INTERLFOW,
+        ),
+        message="v2_interflow.porosity is recommended as fallback value when using a porosity_file.",
     ),
     QueryCheck(
         error_code=417,
         column=models.Interflow.porosity_layer_thickness,
         invalid=Query(models.Interflow).filter(
-            models.Interflow.global_settings != None,
+            interflow_filter,
             (models.Interflow.porosity_layer_thickness == None)
             | (models.Interflow.porosity_layer_thickness <= 0),
             models.Interflow.interflow_type
@@ -957,41 +1175,101 @@ CHECKS += [
         error_code=418,
         column=models.Interflow.impervious_layer_elevation,
         invalid=Query(models.Interflow).filter(
-            models.Interflow.global_settings != None,
+            interflow_filter,
             models.Interflow.impervious_layer_elevation == None,
             models.Interflow.interflow_type != constants.InterflowType.NO_INTERLFOW,
         ),
-        message=f"the impervious layer elevation (v2_interflow.impervious_layer_elevation) cannot be null when "
-        f"v2_interflow.interflow_type is not {constants.InterflowType.NO_INTERLFOW}",
+        message="v2_interflow.impervious_layer_elevation cannot be null",
     ),
     QueryCheck(
         error_code=419,
         column=models.Interflow.hydraulic_conductivity,
         invalid=Query(models.Interflow).filter(
-            models.Interflow.global_settings != None,
-            (models.Interflow.hydraulic_conductivity == None)
-            | (models.Interflow.hydraulic_conductivity < 0),
+            interflow_filter,
+            models.Interflow.hydraulic_conductivity == None,
             is_none_or_empty(models.Interflow.hydraulic_conductivity_file),
             models.Interflow.interflow_type != constants.InterflowType.NO_INTERLFOW,
         ),
-        message="v2_interflow.hydraulic_conductivity cannot be null or negative when no hydraulic conductivity file is supplied.",
+        message="v2_interflow.hydraulic_conductivity must be defined when not using a hydraulic_conductivity_file.",
+    ),
+    QueryCheck(
+        error_code=419,
+        level=CheckLevel.WARNING,
+        column=models.Interflow.hydraulic_conductivity,
+        invalid=Query(models.Interflow).filter(
+            interflow_filter,
+            models.Interflow.hydraulic_conductivity == None,
+            ~is_none_or_empty(models.Interflow.hydraulic_conductivity_file),
+            models.Interflow.interflow_type != constants.InterflowType.NO_INTERLFOW,
+        ),
+        message="v2_interflow.hydraulic_conductivity is recommended as fallback value when using a hydraulic_conductivity_file.",
     ),
     RangeCheck(
         error_code=420,
         column=models.GroundWater.phreatic_storage_capacity,
-        filters=models.GroundWater.global_settings != None,
+        filters=groundwater_filter,
         min_value=0,
         max_value=1,
     ),
-    QueryCheck(
+    RangeCheck(
         error_code=421,
         column=models.GroundWater.groundwater_hydro_connectivity,
-        invalid=Query(models.GroundWater).filter(
-            models.GroundWater.global_settings != None,
-            (models.GroundWater.groundwater_hydro_connectivity < 0),
-            is_none_or_empty(models.GroundWater.groundwater_hydro_connectivity_file),
+        filters=groundwater_filter,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=422,
+        column=models.SimpleInfiltration.max_infiltration_capacity,
+        filters=infiltration_filter,
+        min_value=0,
+    ),
+    QueryCheck(
+        error_code=423,
+        level=CheckLevel.WARNING,
+        column=models.SimpleInfiltration.max_infiltration_capacity,
+        invalid=Query(models.SimpleInfiltration).filter(
+            infiltration_filter,
+            models.SimpleInfiltration.max_infiltration_capacity == None,
+            ~is_none_or_empty(models.SimpleInfiltration.max_infiltration_capacity_file),
         ),
-        message="the global hydro connectivity (v2_groundwater.groundwater_hydro_connectivity) should be >=0 when not using an hydro connectivity file.",
+        message="v2_simple_infiltration.max_infiltration_capacity is recommended as fallback value when using an max_infiltration_capacity_file.",
+    ),
+    RangeCheck(
+        error_code=424,
+        column=models.Interflow.hydraulic_conductivity,
+        filters=(interflow_filter)
+        & (models.Interflow.interflow_type != constants.InterflowType.NO_INTERLFOW),
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=425,
+        column=models.GroundWater.initial_infiltration_rate,
+        filters=groundwater_filter,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=426,
+        column=models.GroundWater.equilibrium_infiltration_rate,
+        filters=groundwater_filter,
+        min_value=0,
+    ),
+    RangeCheck(
+        error_code=427,
+        column=models.GroundWater.infiltration_decay_period,
+        filters=groundwater_filter,
+        min_value=0,
+        left_inclusive=False,
+    ),
+    QueryCheck(
+        error_code=428,
+        level=CheckLevel.WARNING,
+        column=models.GroundWater.groundwater_hydro_connectivity,
+        invalid=Query(models.GroundWater).filter(
+            groundwater_filter,
+            (models.GroundWater.groundwater_hydro_connectivity == None),
+            ~is_none_or_empty(models.GroundWater.groundwater_hydro_connectivity_file),
+        ),
+        message="v2_groundwater.groundwater_hydro_connectivity is recommended as fallback value when using a groundwater_hydro_connectivity_file.",
     ),
 ]
 
@@ -1088,74 +1366,201 @@ CHECKS += [
 ]
 
 
-# 07xx: FILE EXISTENCE
+# 07xx: RASTERS
+RASTER_COLUMNS_FILTERS = [
+    (models.GlobalSetting.dem_file, first_setting_filter),
+    (models.GlobalSetting.frict_coef_file, first_setting_filter),
+    (models.GlobalSetting.interception_file, first_setting_filter),
+    (models.Interflow.porosity_file, interflow_filter),
+    (
+        models.Interflow.hydraulic_conductivity_file,
+        interflow_filter,
+    ),
+    (
+        models.SimpleInfiltration.infiltration_rate_file,
+        infiltration_filter,
+    ),
+    (
+        models.SimpleInfiltration.max_infiltration_capacity_file,
+        infiltration_filter,
+    ),
+    (
+        models.GroundWater.groundwater_impervious_layer_level_file,
+        groundwater_filter,
+    ),
+    (
+        models.GroundWater.phreatic_storage_capacity_file,
+        groundwater_filter,
+    ),
+    (
+        models.GroundWater.equilibrium_infiltration_rate_file,
+        groundwater_filter,
+    ),
+    (
+        models.GroundWater.initial_infiltration_rate_file,
+        groundwater_filter,
+    ),
+    (
+        models.GroundWater.infiltration_decay_period_file,
+        groundwater_filter,
+    ),
+    (
+        models.GroundWater.groundwater_hydro_connectivity_file,
+        groundwater_filter,
+    ),
+    (models.GroundWater.leakage_file, groundwater_filter),
+    (models.GlobalSetting.initial_waterlevel_file, first_setting_filter),
+    (
+        models.GlobalSetting.initial_groundwater_level_file,
+        first_setting_filter & (models.GlobalSetting.groundwater_settings_id != None),
+    ),
+]
 CHECKS += [
-    FileExistsCheck(error_code=701, column=models.GlobalSetting.dem_file),
-    FileExistsCheck(error_code=702, column=models.GlobalSetting.frict_coef_file),
-    FileExistsCheck(error_code=703, column=models.GlobalSetting.interception_file),
-    FileExistsCheck(
-        error_code=704,
+    GDALAvailableCheck(
+        error_code=700, level=CheckLevel.WARNING, column=models.GlobalSetting.dem_file
+    )
+]
+CHECKS += [
+    RasterExistsCheck(
+        error_code=701 + i,
+        column=column,
+        filters=filters,
+    )
+    for i, (column, filters) in enumerate(RASTER_COLUMNS_FILTERS)
+]
+CHECKS += [
+    RasterIsValidCheck(
+        error_code=721 + i,
+        column=column,
+        filters=filters,
+    )
+    for i, (column, filters) in enumerate(RASTER_COLUMNS_FILTERS)
+]
+CHECKS += [
+    RasterHasOneBandCheck(
+        error_code=741 + i,
+        level=CheckLevel.WARNING,
+        column=column,
+        filters=filters,
+    )
+    for i, (column, filters) in enumerate(RASTER_COLUMNS_FILTERS)
+]
+CHECKS += [
+    RasterHasProjectionCheck(
+        error_code=761 + i,
+        column=column,
+        filters=filters,
+    )
+    for i, (column, filters) in enumerate(RASTER_COLUMNS_FILTERS)
+]
+CHECKS += [
+    RasterIsProjectedCheck(
+        error_code=779,
+        column=models.GlobalSetting.dem_file,
+        filters=first_setting_filter,
+    ),
+    RasterSquareCellsCheck(
+        error_code=780,
+        column=models.GlobalSetting.dem_file,
+        filters=first_setting_filter,
+    ),
+    RasterRangeCheck(
+        error_code=781,
+        column=models.GlobalSetting.dem_file,
+        filters=first_setting_filter,
+        min_value=-9998.0,
+        max_value=8848.0,
+    ),
+    RasterRangeCheck(
+        error_code=782,
+        column=models.GlobalSetting.frict_coef_file,
+        filters=CONDITIONS["manning"].exists(),
+        min_value=0,
+        max_value=1,
+    ),
+    RasterRangeCheck(
+        error_code=783,
+        column=models.GlobalSetting.frict_coef_file,
+        filters=CONDITIONS["chezy"].exists(),
+        min_value=0,
+    ),
+    RasterRangeCheck(
+        error_code=784,
         column=models.Interflow.porosity_file,
-        filters=models.Interflow.global_settings != None,
+        filters=interflow_filter,
+        min_value=0,
+        max_value=1,
     ),
-    FileExistsCheck(
-        error_code=705,
+    RasterRangeCheck(
+        error_code=785,
         column=models.Interflow.hydraulic_conductivity_file,
-        filters=models.Interflow.global_settings != None,
+        filters=interflow_filter,
+        min_value=0,
     ),
-    FileExistsCheck(
-        error_code=706,
+    RasterRangeCheck(
+        error_code=786,
         column=models.SimpleInfiltration.infiltration_rate_file,
-        filters=models.SimpleInfiltration.global_settings != None,
+        filters=infiltration_filter,
+        min_value=0,
     ),
-    FileExistsCheck(
-        error_code=707,
+    RasterRangeCheck(
+        error_code=787,
         column=models.SimpleInfiltration.max_infiltration_capacity_file,
-        filters=models.SimpleInfiltration.global_settings != None,
+        filters=infiltration_filter,
+        min_value=0,
     ),
-    FileExistsCheck(
-        error_code=708,
+    RasterRangeCheck(
+        error_code=788,
         column=models.GroundWater.groundwater_impervious_layer_level_file,
-        filters=models.GroundWater.global_settings != None,
+        filters=groundwater_filter,
+        min_value=-9998.0,
+        max_value=8848.0,
     ),
-    FileExistsCheck(
-        error_code=709,
+    RasterRangeCheck(
+        error_code=789,
         column=models.GroundWater.phreatic_storage_capacity_file,
-        filters=models.GroundWater.global_settings != None,
+        filters=groundwater_filter,
+        min_value=0,
+        max_value=1,
     ),
-    FileExistsCheck(
-        error_code=710,
+    RasterRangeCheck(
+        error_code=790,
         column=models.GroundWater.equilibrium_infiltration_rate_file,
-        filters=models.GroundWater.global_settings != None,
+        filters=groundwater_filter,
+        min_value=0,
     ),
-    FileExistsCheck(
-        error_code=711,
+    RasterRangeCheck(
+        error_code=791,
         column=models.GroundWater.initial_infiltration_rate_file,
-        filters=models.GroundWater.global_settings != None,
+        filters=groundwater_filter,
+        min_value=0,
     ),
-    FileExistsCheck(
-        error_code=712,
+    RasterRangeCheck(
+        error_code=792,
         column=models.GroundWater.infiltration_decay_period_file,
-        filters=models.GroundWater.global_settings != None,
+        filters=groundwater_filter,
+        min_value=0,
+        left_inclusive=False,
     ),
-    FileExistsCheck(
-        error_code=713,
+    RasterRangeCheck(
+        error_code=793,
         column=models.GroundWater.groundwater_hydro_connectivity_file,
-        filters=models.GroundWater.global_settings != None,
+        filters=groundwater_filter,
+        min_value=0,
     ),
-    FileExistsCheck(
-        error_code=714,
-        column=models.GroundWater.leakage_file,
-        filters=(models.GroundWater.global_settings != None),
-    ),
-    FileExistsCheck(
-        error_code=715,
+    RasterRangeCheck(
+        error_code=795,
         column=models.GlobalSetting.initial_waterlevel_file,
+        min_value=-9998.0,
+        max_value=8848.0,
     ),
-    FileExistsCheck(
-        error_code=716,
+    RasterRangeCheck(
+        error_code=796,
         column=models.GlobalSetting.initial_groundwater_level_file,
-        filters=(models.GlobalSetting.groundwater_settings_id != None),
+        filters=first_setting_filter
+        & (models.GlobalSetting.groundwater_settings_id != None),
+        min_value=-9998.0,
+        max_value=8848.0,
     ),
 ]
 
